#!/usr/bin/ruby

gem "filename"
autoload :FileName, "filename"
autoload :FileUtils, "fileutils"
require 'optparse'

help_message =<<HELP
Usage: ruby #{File.basename(__FILE__)}
HELP

color = 8
depths = [16, 8]
dpi = nil

begin
  OptionParser.new(help_message) do |opt|
    opt.on("--colors", Integer, "Set the number of colors. Default is #{color}.") do |v|
      color = v
    end
    opt.on("--dpi NUM", Integer, "Set the number of DPI.") do |v|
      dpi = v
    end
    opt.on("--depth NUMS", String, "Set the depths. Default is '#{depths.map(&:to_s).join(" ")}'.") do |v|
      depths = v.split.map(&:to_i)
    end
    opt.parse!(ARGV)
  end
rescue OptionParser::InvalidOption
  $stderr.print <<MES
error: Invalid Option
#{help_message}
MES
  exit(2)
rescue OptionParser::InvalidArgument
  $stderr.print <<MES
error: Invalid Argument
#{help_message}
MES
  exit(2)
end

ARGV.each do |path|
  puts path
  tmp = FileName.create(path, :type => :time, :format => "tmp_%s_%L", :extension => "png", :add => :always, :position => :prefix)
  src = path
  depths.each do |dep|
    convert_args = ["convert", "-depth", dep.to_s, src, tmp]
    unless system(*convert_args)
      puts "[Error] Command 'convert' raises an error."
      p convert_args
      exit(1)
    end
    src = tmp
  end

  ppm = FileName.create(path, :position => :middle, :extension => "ppm")
  convert_args = ["convert", "-colors", color.to_s, src, ppm]
  unless system(*convert_args)
    puts "[Error] Command 'convert' raises an error."
    p convert_args
    exit(1)
  end
  FileUtils.rm(tmp)

  djvu = FileName.create(path, :position => :middle, :extension => "djvu")
  cpaldjvu_args = ["cpaldjvu", "-colors", color.to_s]
  cpaldjvu_args.concat(["-dpi", dpi.to_s]) if dpi
  cpaldjvu_args << ppm << djvu
  unless system(*cpaldjvu_args)
    puts "[Error] Command 'cpaldjvu' raises an error."
    p cpaldjvu
    exit(1)
  end
end

# #!/bin/zsh

# srcfile=$1
# echo $srcfile
# tmpfile=tmp_$srcfile:r.png
# convert -depth 16 $srcfile $tmpfile
# convert -depth  8 $tmpfile $tmpfile
# convert -colors  8 $tmpfile $srcfile:r.ppm
# rm $tmpfile
# cpaldjvu -dpi 400 -colors 8 $srcfile:r.ppm $srcfile:r.djvu
